package de.hft.stuttgart.citydoctor2.healing.math;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.hft.stuttgart.citydoctor2.datastructure.Edge;

public class EdgeGraph {
	
	private Map<Edge, EdgeNode> nodes = new HashMap<>();
	
	public void addEdge(Edge e, List<Edge> edges) {
		if (nodes.containsKey(e)) {
			return;
		}
		EdgeNode node = new EdgeNode(e);
		nodes.put(e, node);
		for (Edge childEdge : getEdgesConnected(e, edges)) {
			EdgeNode childNode = nodes.get(childEdge);
			if (childNode != null && childNode != node) {
				node.addChild(childNode);
				childNode.addChild(node);
			}
		}
	}
	
	private List<Edge> getEdgesConnected(Edge e, List<Edge> edges) {
		List<Edge> result = new ArrayList<>();
		for (Edge potentialEdge : edges) {
			if (potentialEdge.getConnectionPoint(e) != null) {
				result.add(potentialEdge);
			}
		}
		return result;
	}
	
	public List<List<Edge>> getCycles() {
		List<List<Edge>> cycles = new ArrayList<>();
		for (EdgeNode n : nodes.values()) {
			List<Edge> edges = new ArrayList<>();
			n.collectConnectedEdges(edges, null);
			if (edges.size() > 2) {
				cycles.add(edges);
			}
		}
		return cycles;
	}
	

}
